;
; Dynamic Range Increase, V2.4
;
; Martin Egger (martin.egger@gmx.net)
; (C) 2007, Bern, Switzerland
;
; You can find more about Dynamic Range Extension at
; http://www.gimpguru.org/Tutorials/BlendingExposures/
;
; Input image with two layers:
;
;	Upper layer: Darker Image ((under)exposed for highlights)
;	Lower layer: Lighter Image ((over)exposed for shadows)
;
; This script was tested with Gimp 2.4
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 3 of the License, or
; (at your option) any later version.
; 
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
; 
; You should have received a copy of the GNU General Public License
; along with this program; if not, see <http://www.gnu.org/licenses>.
;
; Define the function
;
(define (script-fu-Eg-DynamicRangeIncrease InImage InLayer InOpacity InFlatten)
;
; Save history			
;
	(gimp-image-undo-group-start InImage)
;
	(let*	(
		(OrigLayer (cadr (gimp-image-get-layers InImage)))
		(UpperLayer (aref OrigLayer 0))
		(LowerLayer (aref OrigLayer 1))
		(TempLayer (car (gimp-layer-copy UpperLayer TRUE)))
		)
;
		(gimp-image-add-layer InImage TempLayer -1)
		(if (= (car (gimp-drawable-is-gray TempLayer)) FALSE ) (gimp-desaturate TempLayer))
		(gimp-selection-all InImage)
		(gimp-edit-copy TempLayer)
		(gimp-image-remove-layer InImage TempLayer)
		(let* (
			(UpperLayerMask (car (gimp-layer-create-mask UpperLayer ADD-SELECTION-MASK)))
			)
			(gimp-layer-add-mask UpperLayer UpperLayerMask)
			(gimp-floating-sel-anchor (car (gimp-edit-paste UpperLayerMask FALSE)))
			(gimp-selection-none InImage)
			(gimp-layer-set-edit-mask UpperLayer FALSE)
		)
		(gimp-layer-set-opacity UpperLayer InOpacity)
;
; Flatten the image, if we need to
;
		(cond
			((= InFlatten TRUE) (gimp-image-merge-down InImage UpperLayer CLIP-TO-IMAGE))
			((= InFlatten FALSE) 
				(begin
					(gimp-drawable-set-name UpperLayer "Dark-Overlay")
					(gimp-image-set-active-layer InImage InLayer)
				)
			)
		)
	)
;
; Finish work
;
	(gimp-image-undo-group-end InImage)
	(gimp-displays-flush)
;
)
;
(script-fu-register "script-fu-Eg-DynamicRangeIncrease"
	_"D_ynamic Range Increase"
	"Blend two differently exposed images together thus increasing dynamic range. Upper layer: Darker Image ((under)exposed for highlights). Lower layer: Lighter Image ((over)exposed for shadows)"
	"Martin Egger (martin.egger@gmx.net)"
	"Martin Egger, Bern, Switzerland"
	"07.10.2007"
	"RGB* GRAY*"
	SF-IMAGE	"The Image"		0
	SF-DRAWABLE	"The Layer"		0
	SF-ADJUSTMENT	"Layer Opacity"		'(95.0 1.0 99.0 1.0 0 2 0)
	SF-TOGGLE	"Flatten Image"		FALSE
)
;
(script-fu-menu-register "script-fu-Eg-DynamicRangeIncrease"
			 "<Image>/Photo/Enhancement/Lights & Shadows")
;
