; The GIMP -- an image manipulation program
; Copyright (C) 1995 Spencer Kimball and Peter Mattis
; 
; Extended Split Tone script
; Copyright (c) 2008 Andrey Lebedenko 
; <andrey.lebedenko@gmail.com>
; Tested with GIMP 2.4
;
; Author statement:
;
; This script creates two sets of layers - one for highlights and second for shadows.
; You can then adjust the tone by changing opacity.
;
;; ----------------------------------------------------------------
; 
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 3 of the License, or
; (at your option) any later version.  
; 
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
; 
; You should have received a copy of the GNU General Public License
; along with this program; if not, you can view the GNU General Public
; License version 3 at the web site http://www.gnu.org/licenses/gpl-3.0.html
; Alternatively you can write to the Free Software Foundation, Inc., 675 Mass
; Ave, Cambridge, MA 02139, USA.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


(define (script-fu-ex-split-tone 	theImage
				theLayer
				highMaskGain
				lowMaskGain
				bDarkLayer
	)

; Initialize an undo, so the process can be undone with a single undo
    (gimp-image-undo-group-start theImage)

; Variable init
    (let* (
	(myForeground)
	(myBackground)
	(imageWidth)
	(imageHeight)
	(newLayer)
	(copyHigh_R)
	(copyHigh_G)
	(copyHigh_B)
	(copyLow_R)
	(copyLow_G)
	(copyLow_B)
	(mergedShadows)
	(mergedHighlights)
	(mask)
	(layerRGB)
    )

;Read the current colours
    (set! myForeground (car (gimp-context-get-foreground)))
    (set! myBackground (car (gimp-context-get-background)))

;Select none
    (gimp-selection-none theImage)

;Detect if it is RGB
    (set! layerRGB (car (gimp-drawable-is-rgb theLayer)))

;Change the image RGB if it isn't already
    (if (= layerRGB 0) (gimp-image-convert-rgb theImage))

;Read the image height and width
    (set! imageWidth (car (gimp-image-width theImage)))
    (set! imageHeight (car (gimp-image-height theImage)))

;----- BW ------
    (gimp-desaturate theLayer)

; Add the first set of layers to the image - Low Light

; -----------------------
; ------ LOW LIGHT
; -----------------------

;----- RED -----

; Add one layer and then duplicate it
    (set! copyLow_R (car (gimp-layer-copy theLayer 1)))
    (gimp-image-add-layer theImage copyLow_R -1)

;Rename the layer
    (gimp-drawable-set-name copyLow_R "LowRed")

    ;Add a layer mask
    (set! mask (car (gimp-layer-create-mask copyLow_R ADD-COPY-MASK)))
    (gimp-layer-add-mask copyLow_R mask)
    (gimp-invert mask)
    (gimp-brightness-contrast mask 0 lowMaskGain)

    ;Fill each layer with proper FG colour
    (gimp-context-set-foreground "RED")
    (gimp-drawable-fill copyLow_R 0)
    (gimp-layer-set-mode copyLow_R ADDITION-MODE)

;----- GREEN ------

    (set! copyLow_G (car (gimp-layer-copy theLayer 1)))
    (gimp-image-add-layer theImage copyLow_G -1)

;Rename the layer
    (gimp-drawable-set-name copyLow_G "LowGreen")

    ;Add a layer mask
    (set! mask (car (gimp-layer-create-mask copyLow_G ADD-COPY-MASK)))
    (gimp-layer-add-mask copyLow_G mask)
    (gimp-invert mask)
    (gimp-brightness-contrast mask 0 lowMaskGain)

    ;Fill each layer with proper FG colour
    (gimp-context-set-foreground "GREEN")
    (gimp-drawable-fill copyLow_G 0)
    (gimp-layer-set-mode copyLow_G ADDITION-MODE)

;----- BLUE ------

    (set! copyLow_B (car (gimp-layer-copy theLayer 1)))
    (gimp-image-add-layer theImage copyLow_B -1)

;Rename the layer
    (gimp-drawable-set-name copyLow_B "LowBlue")

    ;Add a layer mask
    (set! mask (car (gimp-layer-create-mask copyLow_B ADD-COPY-MASK)))
    (gimp-layer-add-mask copyLow_B mask)
    (gimp-invert mask)
    (gimp-brightness-contrast mask 0 lowMaskGain)

    ;Fill each layer with proper FG colour
    (gimp-context-set-foreground "BLUE")
    (gimp-drawable-fill copyLow_B 0)
    (gimp-layer-set-mode copyLow_B ADDITION-MODE)

; -----------------------
; ------ HIGH LIGHT
; -----------------------

;----- RED -----

; Add one layer and then duplicate it
    (set! copyLow_R (car (gimp-layer-copy theLayer 1)))
    (gimp-image-add-layer theImage copyLow_R -1)

;Rename the layer
    (gimp-drawable-set-name copyLow_R "HighRed")

    ;Add a layer mask
    (set! mask (car (gimp-layer-create-mask copyLow_R ADD-COPY-MASK)))
    (gimp-layer-add-mask copyLow_R mask)
;    (gimp-invert mask)
    (gimp-brightness-contrast mask 0 lowMaskGain)

    ;Fill each layer with proper FG colour
    (gimp-context-set-foreground "RED")
    (gimp-drawable-fill copyLow_R 0)
    (gimp-layer-set-mode copyLow_R ADDITION-MODE)

;----- GREEN ------

    (set! copyLow_G (car (gimp-layer-copy theLayer 1)))
    (gimp-image-add-layer theImage copyLow_G -1)

;Rename the layer
    (gimp-drawable-set-name copyLow_G "HighGreen")

    ;Add a layer mask
    (set! mask (car (gimp-layer-create-mask copyLow_G ADD-COPY-MASK)))
    (gimp-layer-add-mask copyLow_G mask)
;    (gimp-invert mask)
    (gimp-brightness-contrast mask 0 lowMaskGain)

    ;Fill each layer with proper FG colour
    (gimp-context-set-foreground "GREEN")
    (gimp-drawable-fill copyLow_G 0)
    (gimp-layer-set-mode copyLow_G ADDITION-MODE)

;----- BLUE ------

    (set! copyLow_B (car (gimp-layer-copy theLayer 1)))
    (gimp-image-add-layer theImage copyLow_B -1)

;Rename the layer
    (gimp-drawable-set-name copyLow_B "HighBlue")

    ;Add a layer mask
    (set! mask (car (gimp-layer-create-mask copyLow_B ADD-COPY-MASK)))
    (gimp-layer-add-mask copyLow_B mask)
;    (gimp-invert mask)
    (gimp-brightness-contrast mask 0 lowMaskGain)

    ;Fill each layer with proper FG colour
    (gimp-context-set-foreground "BLUE")
    (gimp-drawable-fill copyLow_B 0)
    (gimp-layer-set-mode copyLow_B ADDITION-MODE)

;------------
; Check for dark layer. Yeah, Luke, go to the dark layer...
; can use copyLow_R variable since it is no longe needed
;------------
    (if (= bDarkLayer TRUE)
      (begin
        (set! copyLow_R (car (gimp-layer-new theImage imageWidth imageHeight RGBA-IMAGE
                                  "Soft Light" 100 SOFTLIGHT-MODE)))
        (gimp-context-set-foreground "BLACK")
        (gimp-drawable-fill copyLow_R 0)
        (gimp-image-add-layer theImage copyLow_R -1)
      )
    )


    ;Finish the undo group for the process
    (gimp-image-undo-group-end theImage)

    ;Set the FG and BG colours back to what they were
    (gimp-context-set-foreground myForeground)
    (gimp-context-set-background myBackground)

    ;Ensure the updated image is displayed now
    (gimp-displays-flush)

    
    )
)


(script-fu-register "script-fu-ex-split-tone"
            _"<Image>/Photo/Effects/Simulations/Extended Split Tone"
            "Turns a B&W image into a split tone image"
            "Andrey Lebedenko"
            "Andrey Lebedenko"
            "10.03.2008"
            "*"
            SF-IMAGE        "Image"     0
            SF-DRAWABLE     "Drawable"  0
            SF-ADJUSTMENT   _"Highlights mask gain:"     '(60 -127 127 5 10 1 0)
	    SF-ADJUSTMENT   _"Shadows mask gain:"      '(60 -127 127 5 10 1 0)
            SF-TOGGLE       _"Create dark layer"	TRUE
)
