;
; Copyright, V1.1
;
; Marian Kyral (mkyral@email.cz)
; (C) 2006, 2008, Frydek-Mistek, Czech
;
; This plugin was tested with Gimp 2.4
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;
; Define the function
;
(define (script-fu-kym-Line-Border InImage InLayer InOuterTotalWidth InOuterTotalHeight InInnerSize InUseSysColors InLineColor InBorderColor InFeather InOuterLine InWorkOnCopy)
;
  (let*
    (
        (TheImage)
        (TheLayer)
        (TheWidth)
        (TheHeight)
        (inner-border-width)
        (inner-border-height)
        (total-border-width)
        (total-border-height)
        (image-width)
        (image-height)
        (TheBackupColor)
        (TheLineColor)
        (TheBorderColor)
    )

        (if (= InWorkOnCopy TRUE)
                           (begin
                             (set! TheImage (car (gimp-image-duplicate InImage)))
                             (gimp-image-undo-disable TheImage)
                             (gimp-selection-none TheImage)
                           )
                           (begin
                             (set! TheImage InImage)
                             (gimp-image-undo-group-start TheImage)
                             (gimp-selection-all TheImage)
                           )
        )

        (set! TheLayer (car (gimp-image-flatten TheImage)))
        (set! TheWidth (car (gimp-image-width TheImage)))
        (set! TheHeight (car (gimp-image-height TheImage)))
        (set! inner-border-width  InInnerSize)
        (set! inner-border-height inner-border-width)
        (set! total-border-width  InOuterTotalWidth)
        (set! total-border-height InOuterTotalHeight)
        (set! image-width (+ TheWidth  (* 2 total-border-width)))
        (set! image-height (+ TheHeight (* 2 total-border-height)))

        (set! TheBackupColor (car (gimp-context-get-foreground)))

        (if (= InUseSysColors TRUE)
            (begin
              (set! TheLineColor (car (gimp-context-get-foreground) ))
              (set! TheBorderColor (car (gimp-context-get-background) ))
            )
            (begin
              (set! TheLineColor InLineColor )
              (set! TheBorderColor InBorderColor )
            )
        )

        (set! TheLayer ( car (gimp-layer-copy TheLayer TRUE)))
        (gimp-drawable-set-name TheLayer "With Border")
;
; Generate the border
;
        (gimp-image-resize TheImage image-width image-height total-border-width total-border-height)
;
        (let*    (
            (BorderLayer (car (gimp-layer-new TheImage image-width image-height RGBA-IMAGE "TempLayer" 100 NORMAL-MODE)))
            )
            (gimp-image-add-layer TheImage BorderLayer -1)
            (gimp-edit-clear BorderLayer)
;
            (gimp-rect-select TheImage 0 0 image-width image-height CHANNEL-OP-REPLACE FALSE 0)
            (gimp-rect-select TheImage total-border-width total-border-height TheWidth TheHeight CHANNEL-OP-SUBTRACT FALSE 0)
            (gimp-palette-set-foreground TheBorderColor)
            (gimp-edit-fill BorderLayer FOREGROUND-FILL)
;

            (cond
                ((> InInnerSize 0)
                    (begin
                  (cond
                      ((= InOuterLine TRUE)
                          (begin
;
; Make the outer border line
;
                              (gimp-rect-select TheImage (* inner-border-width  1)
                                             (* inner-border-height 1)
                                             (- image-width  (* inner-border-width  2))
                                             (- image-height (* inner-border-height 2))
                                    CHANNEL-OP-REPLACE InFeather (* 1.2 inner-border-width)
                  )
                              (gimp-rect-select TheImage total-border-width total-border-height TheWidth TheHeight CHANNEL-OP-SUBTRACT FALSE 0)
                              (gimp-palette-set-foreground TheLineColor)
                              (gimp-edit-fill BorderLayer FOREGROUND-FILL)

                              (gimp-rect-select TheImage (* inner-border-width  2)
                                             (* inner-border-height 2)
                                             (- image-width  (* inner-border-width  4))
                                             (- image-height (* inner-border-height 4))
                                    CHANNEL-OP-REPLACE InFeather (* 1.2 inner-border-width)
                  )
                              (gimp-rect-select TheImage total-border-width total-border-height TheWidth TheHeight CHANNEL-OP-SUBTRACT FALSE 0)
                              (gimp-palette-set-foreground TheBorderColor)
                              (gimp-edit-fill BorderLayer FOREGROUND-FILL)
                          )
                      )
                  )
;
; Make the inner border line
;
                        (gimp-rect-select TheImage (- total-border-width (* inner-border-width 2))
                                       (- total-border-height (* inner-border-height 2))
                                       (- image-width  (- (* total-border-width 2) (* inner-border-width 4)) )
                                       (- image-height (- (* total-border-height 2) (* inner-border-height 4)) )
                              CHANNEL-OP-REPLACE InFeather (* 1.2 inner-border-width)
            )
                        (gimp-rect-select TheImage total-border-width total-border-height TheWidth TheHeight CHANNEL-OP-SUBTRACT FALSE 0)
                        (gimp-palette-set-foreground TheLineColor)
                        (gimp-edit-fill BorderLayer FOREGROUND-FILL)

                        (gimp-rect-select TheImage (- total-border-width inner-border-width)
                                       (- total-border-height inner-border-height)
                                       (- image-width  (- (* total-border-width  2) (* inner-border-width  2)))
                                       (- image-height (- (* total-border-height 2) (* inner-border-height 2)))
                              CHANNEL-OP-REPLACE InFeather (* 1.2 inner-border-width)
            )
                        (gimp-rect-select TheImage total-border-width total-border-height TheWidth TheHeight CHANNEL-OP-SUBTRACT FALSE 0)
                        (gimp-palette-set-foreground TheBorderColor)
                        (gimp-edit-fill BorderLayer FOREGROUND-FILL)
                    )
                )
            )

            (gimp-image-merge-down TheImage BorderLayer CLIP-TO-IMAGE)
        )
;
        (gimp-selection-none InImage)
        (if (= InWorkOnCopy TRUE)
            (begin  (gimp-image-clean-all TheImage)
                    (gimp-display-new TheImage)
                    (gimp-image-undo-enable TheImage)
            )
            (gimp-image-undo-group-end TheImage)
        )
    (gimp-palette-set-foreground TheBackupColor)
    )
;
; Finish work
;
    (gimp-displays-flush)
;
)
;
; Register the function with the GIMP
;
(script-fu-register
    "script-fu-kym-Line-Border"
    "<Image>/Photo/Border/Line Border"
    "Generate a line border around an image."
    "Marian Kyral (mkyral@email.cz)"
    "2006, 2008, Marian Kyral, Frydek-Mistek, Czech"
    "21.06.2006, 06.01.2008"
    "RGB*,GRAY*"
    SF-IMAGE    "The Image"    0
    SF-DRAWABLE    "The Layer"    0
    SF-ADJUSTMENT     "Total border size (width in pixels)" '(25 1 400 1 0 2 0)
    SF-ADJUSTMENT     "Total border size (height in pixels)" '(25 1 400 1 0 2 0)
    SF-ADJUSTMENT      "Line size (in pixels)" '(1 0 50 1.0 0 2 0)
    SF-TOGGLE "Use actual palette colors" FALSE
    SF-COLOR "Line color" '(0 0 0)
    SF-COLOR "Border color" '(255 255 255)
    SF-TOGGLE "Feather inner line" FALSE
    SF-TOGGLE "Make outer line" FALSE
    SF-TOGGLE "Work on copy" FALSE
)
;
