;
; Shadow Recovery, V2.1
;
; Martin Egger (martin.egger@gmx.net)
; (C) 2005, Bern, Switzerland
;
; This plugin was tested with Gimp 2.2
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
; 
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
; 
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;
; modified by Paul Sherman
; 11/23/2007
; renamed to recover_shadows.scm and Menu item adapted
; 12/14/2007
; updated to flatten image prior to use (warning given in parameter dialog)
; to prevent errors when working on multi-layer images
; also selects all of image (as plugin is not set up to deal with them...)
; distributed by gimphelp.org
;
(define (script-fu-recover_shadows InImage InLayer InMethod InOpacity InFlatten)
	; Save history			
	(gimp-image-undo-group-start InImage)
  	(gimp-selection-all InImage)				; Select whole image
  	(set! InLayer (car(gimp-image-flatten InImage)))	; Set the Layer to whole image flattened on one level

	(let*	(
		(CopyLayer (car (gimp-layer-copy InLayer TRUE)))
		(ShadowLayer (car (gimp-layer-copy InLayer TRUE)))
		)

		; Create new layer and add it to the image

		(gimp-image-add-layer InImage CopyLayer -1)
		(gimp-layer-set-mode CopyLayer ADDITION-MODE)
		(gimp-layer-set-opacity CopyLayer InOpacity)
		(gimp-image-add-layer InImage ShadowLayer -1)
		(if (eqv? (car (gimp-drawable-is-rgb ShadowLayer)) TRUE) (gimp-desaturate ShadowLayer))
		(gimp-invert ShadowLayer)
		(let*	(
			(CopyMask (car (gimp-layer-create-mask CopyLayer ADD-WHITE-MASK)))
			(ShadowMask (car (gimp-layer-create-mask ShadowLayer ADD-WHITE-MASK)))
			)
			(gimp-layer-add-mask CopyLayer CopyMask)
			(gimp-layer-add-mask ShadowLayer ShadowMask)
			(gimp-selection-all InImage)
			(gimp-edit-copy ShadowLayer)
			(gimp-floating-sel-anchor (car (gimp-edit-paste CopyMask TRUE)))
			(gimp-floating-sel-anchor (car (gimp-edit-paste ShadowMask TRUE)))
		)
		(gimp-layer-set-mode ShadowLayer OVERLAY-MODE)
		(gimp-layer-set-opacity ShadowLayer InOpacity)
		(if (= InMethod 0) (gimp-image-remove-layer InImage CopyLayer))

		; Flatten the image, if we need to

		(cond
			((= InFlatten TRUE)
				(begin
					(if (= InMethod 1) (gimp-image-merge-down InImage CopyLayer CLIP-TO-IMAGE))
					(gimp-image-merge-down InImage ShadowLayer CLIP-TO-IMAGE)
				)
			)
			((= InFlatten FALSE) 
				(begin
					(if (= InMethod 1) (gimp-drawable-set-name CopyLayer "Shadowfree strong"))
					(gimp-drawable-set-name ShadowLayer "Shadowfree normal")
				)
			)
		)
	)
;
; Finish work
;
	(gimp-image-undo-group-end InImage)
	(gimp-displays-flush)
;
)
;
(script-fu-register "script-fu-recover_shadows"
	"<Image>/Photo/Enhancement/Lights & Shadows/Recover Shadows"
	"Lighten-up Shadows"
	"Martin Egger (martin.egger@gmx.net)"
	"2005, Martin Egger, Bern, Switzerland"
	"2.06.2005"
	"RGB* GRAY*"
	SF-IMAGE	"The Image"	0
	SF-DRAWABLE	"The Layer"	0
	SF-OPTION 	"Shadow Recovery Method" 
			'( 
						"Normal"
						"Strong"
			)
	SF-ADJUSTMENT	"Layer Opacity"	'(60.0 1.0 100.0 1.0 0 2 0)
	SF-TOGGLE	"\n\nFlatten Image when complete\n(NOTE: this filter will automatically flatten\n           the image prior to running.)"	FALSE
)
;
